TACHYON

\ EXTEND.fth - NOW INCLUDES I2C ROUTINES and backs-up after loading }

\ ******************* TACHYON FORTH KERNEL EXTENSIONS + I2C BUS ROUTINES *********************

\ UPDATED: 120806.0017

HEX

: CLKFREQ ( -- freq )        0 @ ;

: >CFA  ( atr -- cfa ) 2 + C@ 2* 2* XCALLS + W@ ;

\ Quick WORDS - a simpler and more compact listing of the current dictionary

: QWORDS

        CR

        0 names W@

        BEGIN

        SWAP OVER STRLEN + 1+ DUP #80 > IF CR DROP OVER STRLEN 1+ THEN

        SWAP

        DUP .STR BL EMIT

        NFA>CFA 2+

        SWAP 1+ SWAP

        DUP C@ 0=

        UNTIL

        2DROP CR

        ;

\ Serial output - transmit asynchronous data

{ Replaced with separate BAUD and SEROUT methods for efficiency and speed

: SEROUT ( data baud pin -- ) \ Baudrate can be set up to 400K baud

        MASK DUP OUTSET ROT        ( baud mask data )

        $100 OR 2*                ( baud mask data )

        ROT #10 SWAP CLKFREQ SWAP / DELTA ( mask data )

        FOR SHROUT WAITCNT NEXT

        2DROP

        ;

}

: BAUD ( baud -- ) CLKFREQ SWAP / baudcnt ! ;

: SEROUT ( data pin -- )

        MASK DUP OUTSET SWAP        ( mask data )

        $100 OR 2*                \ ( baud mask data ) stop and start bits

        #10 baudcnt @ DELTA ( mask data )

        FOR SHROUT WAITCNT NEXT

        2DROP

        ;

9600d BAUD

: IMMEDIATE        names W@ DUP STRLEN + DUP C@ $20 XOR SWAP C! ;

\ Change output device back to the default CONSOLE

: CON        0 uemit W! ;

\ ******************************* I2C BUS **************************

\ I/O Constants

: SCL                28d MASK ;

: SDA                29d MASK ;

: SDA?                29d INPUT ;

\ I2C START

: I2CST        SDA INPUTS SCL OUTSET SDA OUTCLR SCL OUTCLR ;

\ I2C STOP

: I2CSP        SDA OUTCLR SCL OUTSET SDA INPUTS ;

\ This routine runs at an I2C speed of 125kHz

: I2C!         ( data -- flg ) \ write a byte to the I2C bus and return with the ack (0=ack)

                #24 REV SCL SWAP SDA DUP OUTSET SWAP 8

                FOR                ( scl sda data )

                  SHROUT        ( scl sda data/ )

                  3RD OUTSET 3RD OUTCLR                \ clock

                  NEXT

                DROP INPUTS                ( float SDA )

                DUP OUTSET SDA? SWAP OUTCLR ( ack clock )

                ;

\ CLKDAT ( sdat scnt -- data )  ( L: miso mosi sck )

: I2C@        ( ack -- data )

                SDA INPUTS SCL 0  ( ack sck dat )

                8 FOR OVER OUTSET 2* SDA? - OVER OUTCLR NEXT

                ROT 0= IF SDA OUTCLR THEN OVER OUTSET SWAP OUTCLR SDA INPUTS

                  ;

\ EEPROM addressing

: @EE ( addr -- flg )

        I2CST $A0 I2C! OVER 8 SHR I2C! OR SWAP I2C! OR

        ;

\ Switch EEPROM to read mode

: EERD ( -- flg  )

        I2CST $A1 I2C!

        ;

\ last byte read and stop

: ENDRD        1 I2C@ I2CSP ;

\ Hex dump of EEPROM

: EDUMP ( addr cnt -- )

        I2CSP OVER @EE EERD OR

        IF 2DROP ." BAD RESPONSE FROM EEPROM "

        ELSE

        ADO I $0F AND 0= IF CR I .WORD ." : " THEN

          0 I2C@ .BYTE BL EMIT

          LOOP

        ENDRD DROP

        THEN

        ;

\ Store byte to EEPROM

: EE!                ( byte addr -- ) @EE SWAP I2C! OR I2CSP ;

\ Fetch byte from EEPROM

: EE@                ( addr -- byte ) @EE SWAP EERD DROP ENDRD ;

\ Will backup 32K to EEPROM in 4.963 seconds

\ Save a block of RAM to EEPROM

: ESAVE ( ram eeprom cnt -- )

        ROT SWAP ADO

          BEGIN DUP @EE 0= UNTIL

          0 I $40 ADO I C@ I2C! OR LOOP I2CSP IF $0D EMIT ." FAIL @" I .WORD THEN

          SPINNER

          $40 + $40

        +LOOP DROP

        ;

\  Will load 32K from EEPROM in  4.325sec

\ Load a block of EEPROM to RAM

: ELOAD ( eeprom ram cnt -- )

        ROT BEGIN @EE 0= UNTIL EERD DROP

        ADO 0 I2C@ I C! LOOP

        ENDRD DROP

;

( Scan I2C bus for devices and report )

: I2CBUS

        I2CSP

        $100 0 DO I2CST I I2C! 0=

          IF CR ." Device at " I .WORD SPACE SPACE

          I2CST I 1+ I2C! DROP 8 FOR 0 I2C@ .BYTE SPACE NEXT ENDRD DROP

          THEN

        2 +LOOP

        ;

: VERIFY ( ram cnt -- )

        OVER @EE DROP EERD DROP

        ADO 0 I2C@ DUP I C@ <> IF CR I .WORD ." : " I C@ .BYTE SPACE .BYTE

        ELSE DROP THEN LOOP

        ENDRD DROP

        ;

\ Backup the current Tachyon Forth session

: BACKUP

        XCALLS $3F ANDN DUP $840  ESAVE

        0 REG $3F ANDN DUP $100 ESAVE

        FREE DUP $8000 OVER - ESAVE

        ;



END

errors C@ ?EXIT BACKUP
